/**
 * File: tournament-graph-test.cpp
 * -------------------------------
 * Test framework exercising the crownTournamentKings function 
 * and its ability to correctly identify all of the tournament 
 * kings in a tournament graph.
 */

#include <iostream>
#include "console.h"
#include "random.h"
#include "strlib.h"
#include "graphtypes.h"
#include "simpio.h"
#include "tokenscanner.h"
#include "tournament-graph-kings.h"

static const double kUpsetProbability = 0.01;
static void generateTournamentFile(const string& tournamentFileName, int numPlayers) {
	ofstream outfile(tournamentFileName.c_str());
	outfile << numPlayers << endl;
	for (int node1 = 1; node1 < numPlayers; node1++) {
		for (int node2 = node1 + 1; node2 <= numPlayers; node2++) {
			int winner = node1;
			int loser = node2;
			if (randomChance(kUpsetProbability)) {
				swap(winner, loser);
			}
			
			outfile << winner << " " << loser << endl;
		}
	}
}

static void buildGraph(const string& tournamentFileName, SimpleGraph& graph) {
	ifstream infile(tournamentFileName.c_str());
	string line;
	getline(infile, line);
	int numPlayers = stringToInteger(line);
	for (int n = 1; n <= numPlayers; n++) {
		Node *node = new Node;
		node->name = integerToString(n);
		graph.nodes += node;
		graph.nodeMap[node->name] = node;
	}

	TokenScanner scanner;
	scanner.setInput(infile);
	scanner.ignoreWhitespace();
	for (int match = 1; match <= numPlayers * (numPlayers - 1) / 2; match++) {
		Node *winningNode = graph.nodeMap[scanner.nextToken()];
		Node *losingNode = graph.nodeMap[scanner.nextToken()];
		Arc *arc = new Arc;
		arc->start = winningNode;
		arc->finish = losingNode;
		graph.arcs += arc;
		winningNode->arcs += arc;
	}
}

static void postResults(ostream& out, int numPlayers, Set<Node *>& tournamentKings) {
	out << "The tournament of size " << numPlayers << " has this many kings: "
        << tournamentKings.size() << endl;
	Set<int> playerIDs;
	foreach (Node *king in tournamentKings) {
		playerIDs += stringToInteger(king->name);
	}
	
	if (playerIDs.size() == 0) {
		out << "There are no tournament kings." << endl;
	} else if (playerIDs.size() == 1) {
		out << "Player " << playerIDs.first() << " is the only tournament king." << endl;
	} else {
		out << "Players ";
		int numKings = playerIDs.size();
		string word = numKings == 2 ? "both" : "all";
		Set<int>::iterator iter = playerIDs.begin();
		for (int i = 0; i < numKings - 2; i++) {
			out << *iter++ << ", ";
		}
		
		int nextToLastKing = *iter++;
		int lastKing = *iter++;
		out << nextToLastKing << " and " << lastKing << " are " << word << " tournament kings." << endl;
	}
}

static bool filesAreIdentical(const string& fileNameOne, const string& fileNameTwo) {
	ifstream infile1(fileNameOne.c_str());
	ifstream infile2(fileNameTwo.c_str());

	while (true) {
		string line1;
		string line2;
		getline(infile1, line1);
		getline(infile2, line2);
		if (infile1.fail() && infile2.fail()) return true;
		if (infile1.fail() || infile2.fail()) return false;
		if (line1 != line2) return false;
	}
}

static const bool kGeneratingTestCases = false;
static bool testTournamentGraph(int numPlayers) {
	string tournamentFileName =
        "test-files/tournament-graphs/touranment-" + integerToString(numPlayers) + ".txt";
	if (kGeneratingTestCases) {
		generateTournamentFile(tournamentFileName, numPlayers);
	}
	
	SimpleGraph graph;
	buildGraph(tournamentFileName, graph);
	Set<Node *> tournamentKings = crownTournamentKings(graph);
	
	string tournamentKingsSolutionFileName =
        "test-files/tournament-graphs/touranment-kings-solution-" + integerToString(numPlayers) + ".txt";
	if (kGeneratingTestCases) {
		ofstream outfile(tournamentKingsSolutionFileName.c_str());
		postResults(outfile, numPlayers, tournamentKings);
	}
	
	string tournamentKingsTrialFileName =
        "test-files/tournament-graphs/touranment-kings-trial-" + integerToString(numPlayers) + ".txt";
	ofstream outfile(tournamentKingsTrialFileName.c_str());
	postResults(outfile, numPlayers, tournamentKings);
	return filesAreIdentical(tournamentKingsTrialFileName, tournamentKingsSolutionFileName);
}

static const int kNumGraphs = 200;
int main() {
	for (int numPlayers = 2; numPlayers <= kNumGraphs; numPlayers++) {
		if (!testTournamentGraph(numPlayers)) {
			cout << "crownTournamentKings FAILED for a test tournament with " << numPlayers << " players!" << endl;
			cout << "Aborting..." << endl;
			break;
		}
		cout << "Tournament graph of size " << numPlayers << ": PASSED" << endl;
	}
	
    cout << "All done!!! " << endl;
	return 0;
}
